#!/usr/bin/env python3
"""
Cylinders with size distribution
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm

def f_particle():
    # Define materials
    material_Particle = ba.RefractiveMaterial("Particle", 0.0006, 2e-08)
    material_Vacuum = ba.RefractiveMaterial("Vacuum", 0, 0)

    # Define form factors
    ff = ba.Cylinder(5*nm, 5*nm)

    # Define particles
    particle = ba.Particle(material_Particle, ff)

    return particle

def get_sample():
    """
    Return a sample with cylinders on a substrate.
    The cylinders have a Gaussian size distribution.
    """


    # Define particles with parameter following a distribution
    distr_1 = ba.DistributionGaussian(5*nm, 1*nm)
#    par_distr_1 = ba.ParameterDistribution("/Particle/Cylinder/Radius",
#                                           distr_1, 100, 2)
    particle_distrib = ba.ParticleDistribution(f_particle)

    # Define particle layouts
    layout = ba.ParticleLayout()
    layout.addParticle(particle_distrib.draw())
    layout.setTotalParticleSurfaceDensity(0.01)

    # Define layers
    layer = ba.Layer(material_Vacuum)
    layer.addLayout(layout)

    # Define sample
    sample = ba.MultiLayer()
    sample.addLayer(layer)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, ba.Direction(0.2*deg, 0))
    detector = ba.SphericalDetector(bp.simargs['n'], 2*deg, 1*deg, 1*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    bp.parse_args(sim_n=200)
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_simulation_result(result)
