//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Job/FitParameterLinkItem.cpp
//! @brief     Implements class FitParameterLinkItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Job/FitParameterLinkItem.h"
#include "GUI/Support/XML/UtilXML.h"

namespace {

namespace Tag {

const QString Link("Link");
const QString Title("Title");
const QString LinkItem("LinkItem");

} // namespace Tag

namespace Attrib {

const QString value("value");

} // namespace Attrib

} // namespace


LinkItem::LinkItem(QObject* parent)
    : QObject(parent)
    , m_link("")
{
    setObjectName("Link");
}

QString LinkItem::link() const
{
    return m_link;
}

void LinkItem::setLink(QString link)
{
    m_link = link;
}

void LinkItem::writeTo(QXmlStreamWriter* writer) const
{
    // link
    writer->writeStartElement(Tag::Link);
    GUI::Session::XML::writeAttribute(writer, Attrib::value, m_link);
    writer->writeEndElement();
}

void LinkItem::readFrom(QXmlStreamReader* reader)
{
    while (reader->readNextStartElement()) {

        // link
        if (reader->name() == Tag::Link) {
            GUI::Session::XML::readAttribute(reader, Attrib::value, &m_link);
            GUI::Session::XML::gotoEndElementOfTag(reader, Tag::Link);

        } else
            reader->skipCurrentElement();
    }
}

//------------------------------------------------------------------------------------------------

FitParameterLinkItem::FitParameterLinkItem(QObject* parent)
    : QObject(parent)
    , m_title("")
    , m_linkItem(new LinkItem(this))
{
    setObjectName("FitParameterLink");
}

QString FitParameterLinkItem::link() const
{
    return m_linkItem->link();
}

void FitParameterLinkItem::setLink(const QString& link)
{
    m_linkItem->setLink(link);
}

QString FitParameterLinkItem::title() const
{
    return m_title;
}

void FitParameterLinkItem::setTitle(const QString& title)
{
    m_title = title;
}

LinkItem* FitParameterLinkItem::linkItem() const
{
    return m_linkItem;
}

void FitParameterLinkItem::writeTo(QXmlStreamWriter* writer) const
{
    // title
    writer->writeStartElement(Tag::Title);
    GUI::Session::XML::writeAttribute(writer, Attrib::value, m_title);
    writer->writeEndElement();

    // link item
    writer->writeStartElement(Tag::LinkItem);
    m_linkItem->writeTo(writer);
    writer->writeEndElement();
}

void FitParameterLinkItem::readFrom(QXmlStreamReader* reader)
{
    while (reader->readNextStartElement()) {

        // title
        if (reader->name() == Tag::Title) {
            GUI::Session::XML::readAttribute(reader, Attrib::value, &m_title);
            GUI::Session::XML::gotoEndElementOfTag(reader, Tag::Title);

            // link item
        } else if (reader->name() == Tag::LinkItem) {
            m_linkItem->readFrom(reader);
            GUI::Session::XML::gotoEndElementOfTag(reader, Tag::LinkItem);

        } else
            reader->skipCurrentElement();
    }
}
