//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/CoreAndShellItem.h
//! @brief     Defines class CoreAndShellItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_COREANDSHELLITEM_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_COREANDSHELLITEM_H

#include "GUI/Model/Sample/ItemWithParticles.h"
#include <memory>

class CoreAndShell;
class ParticleItem;
class MaterialItems;

class CoreAndShellItem : public ItemWithParticles {
public:
    CoreAndShellItem(const MaterialItems* materials);
    void serialize(Streamer& s) override;

    std::unique_ptr<CoreAndShell> createCoreAndShell() const;

    ParticleItem* core() const;
    void clearCore();
    ParticleItem* createCore(const MaterialItems* materials);

    ParticleItem* shell() const;
    void clearShell();
    ParticleItem* createShell(const MaterialItems* materials);

    QVector<ItemWithParticles*> containedItemsWithParticles() const override;

private:
    std::unique_ptr<ParticleItem> m_core;
    std::unique_ptr<ParticleItem> m_shell;
    const MaterialItems* m_materials;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_COREANDSHELLITEM_H
