//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/LayerRoughnessItems.cpp
//! @brief     Implements classes LayerRoughnessItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/LayerRoughnessItems.h"
#include "GUI/Support/XML/Serialize.h"

namespace {
namespace Tag {

const QString Sigma("Sigma");
const QString Hurst("Hurst");
const QString LateralCorrelationLength("LateralCorrelationLength");

} // namespace Tag
} // namespace

LayerBasicRoughnessItem::LayerBasicRoughnessItem()
{
    m_sigma.init("Sigma", "rms of the roughness", 1.0, Unit::nanometer, "sigma");
    m_hurst.init("Hurst",
                 "Hurst parameter which describes how jagged the interface,\n "
                 "dimensionless [0.0, 1.0], where 0.0 gives more spikes, \n1.0 more smoothness.",
                 0.3, Unit::unitless, 3 /* decimals */, RealLimits::limited(0.0, 1.0), "hurst");
    m_lateralCorrelationLength.init("Correlation length",
                                    "Lateral correlation length of the roughness", 5.0,
                                    Unit::nanometer, "corrLen");
}

void LayerBasicRoughnessItem::serialize(Streamer& s)
{
    s.assertVersion(0);
    Serialize::rwProperty(s, Tag::Sigma, m_sigma);
    Serialize::rwProperty(s, Tag::Hurst, m_hurst);
    Serialize::rwProperty(s, Tag::LateralCorrelationLength, m_lateralCorrelationLength);
}
