//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Particle/ReParticle.h
//! @brief     Defines interface class ReParticle.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_PARTICLE_REPARTICLE_H
#define BORNAGAIN_RESAMPLE_PARTICLE_REPARTICLE_H

#include "Base/Vector/RotMatrix.h"
#include "Resample/Particle/IReParticle.h"
#include <heinz/Vectors3D.h>
#include <memory>

class IFormFactor;
class Material;

//! A reprocessed simple particle, with shape m_ff.

class ReParticle : public IReParticle {
public:
    ReParticle(const IFormFactor& ff);
    ~ReParticle() override;

    ReParticle* clone() const override;

    //! Sets the material of the scatterer
    void setMaterial(const Material& material);

    //! Sets the ambient material
    void setAmbientMaterial(const Material& ambient_material) override;

    void setPosition(const R3& position);
    void setRotMatrix(const RotMatrix& rotMatrix);

    complex_t theFF(const WavevectorInfo& wavevectors) const override;
    SpinMatrix thePolFF(const WavevectorInfo& wavevectors) const override;

    double volume() const override;

    double radialExtension() const override;

    double bottom_Z(const IRotation* rotation) const override;

    double top_Z(const IRotation* rotation) const override;

    const IFormFactor* formfactor_at_bottom() const;

    virtual complex_t formfactor_at_bottom(C3 q) const;

protected:
    std::unique_ptr<const IFormFactor> m_ff;

    // private:
    ReParticle(const std::optional<size_t>& i_layer, IFormFactor* ff, const Material* material,
               const Material* ambient_material, const R3* position, const RotMatrix* rotMatrix);

    std::unique_ptr<const Material> m_material;
    std::unique_ptr<const Material> m_ambient_material;
    std::unique_ptr<const R3> m_position;
    std::unique_ptr<const RotMatrix> m_rotMatrix;
};

#endif // BORNAGAIN_RESAMPLE_PARTICLE_REPARTICLE_H
#endif // USER_API
