//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Slice/ZLimits.cpp
//! @brief     Defines class ZLimits.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Resample/Slice/ZLimits.h"
#include "Base/Util/Assert.h"
#include <algorithm>
#include <cmath>

ZLimits::ZLimits(double _min, double _max)
    : m_zmin(_min)
    , m_zmax(_max)
{
    ASSERT(_min < _max);
}

ZLimits::ZLimits()
    : ZLimits(-inf, inf)
{
}

bool ZLimits::isFinite() const
{
    return !std::isinf(m_zmin) && !std::isinf(m_zmax);
}

// TODO sep22: rm all the following if they remain UNUSED

ZLimits ZLimits::enclosingInterval(const ZLimits& left, const ZLimits& right)
{
    return {std::min(left.zBottom(), right.zBottom()), std::max(left.zTop(), right.zTop())};
}

bool operator==(const ZLimits& left, const ZLimits& right)
{
    return (left.zBottom() == right.zBottom() && left.zTop() == right.zTop());
}

bool operator!=(const ZLimits& left, const ZLimits& right)
{
    return !(left == right);
}

std::ostream& operator<<(std::ostream& ostr, const ZLimits& limits)
{
    return ostr << "Lower: " << limits.zBottom() << ", Upper: " << limits.zTop();
}
