//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/Dodecahedron.cpp
//! @brief     Implements class Dodecahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/Dodecahedron.h"
#include <ff/Make.h>

Dodecahedron::Dodecahedron(const std::vector<double> P)
    : IFormFactorPolyhedron(P)
    , m_edge(m_P[0])
{
    validateOrThrow();
}

Dodecahedron::Dodecahedron(double edge)
    : Dodecahedron(std::vector<double>{edge})
{
}

std::string Dodecahedron::validate() const
{
    if (m_edge <= 0)
        return jointError({"nonpositive edge=" + std::to_string(m_edge)});

    pimpl.reset(ff::make::Dodecahedron(m_edge));
    shift(1.113516364411607 * m_edge);
    assert_platonic();

    m_validated = true;
    return "";
}
