//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/PlatonicOctahedron.cpp
//! @brief     Implements class PlatonicOctahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/PlatonicOctahedron.h"
#include "Base/Math/Constants.h"
#include "Base/Math/Functions.h"
#include <ff/Make.h>

PlatonicOctahedron::PlatonicOctahedron(const std::vector<double> P)
    : IFormFactorPolyhedron(P)
    , m_edge(m_P[0])
{
    validateOrThrow();
}

PlatonicOctahedron::PlatonicOctahedron(double edge)
    : PlatonicOctahedron(std::vector<double>{edge})
{
}


std::string PlatonicOctahedron::validate() const
{
    if (m_edge <= 0)
        return jointError({"nonpositive edge=" + std::to_string(m_edge)});

    pimpl.reset(ff::make::Octahedron(m_edge));
    shift(m_edge / sqrt(2));
    assert_platonic();

    m_validated = true;
    return "";
}
