//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/PlatonicTetrahedron.cpp
//! @brief     Implements class PlatonicTetrahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/PlatonicTetrahedron.h"
#include "Base/Math/Constants.h"
#include "Base/Math/Functions.h"
#include <ff/Make.h>

PlatonicTetrahedron::PlatonicTetrahedron(const std::vector<double> P)
    : IFormFactorPolyhedron(P)
    , m_edge(m_P[0])
{
    validateOrThrow();
}

PlatonicTetrahedron::PlatonicTetrahedron(double edge)
    : PlatonicTetrahedron(std::vector<double>{edge})
{
}

std::string PlatonicTetrahedron::validate() const
{
    if (m_edge <= 0)
        return jointError({"nonpositive edge=" + std::to_string(m_edge)});

    pimpl.reset(ff::make::Tetrahedron(m_edge));
    shift(sqrt(2. / 3) / 4 * m_edge);
    assert_platonic();

    m_validated = true;
    return "";
}
